
"""
    Impay ident
    ~~~~~~~~~~~~~~~~

    :copyright: (c) 2015 THE BEATPACKING COMPANY
    :license: MIT License
"""

import pkg_resources
VERSION = pkg_resources.resource_string('impayident', 'version.txt').strip()

from .api import cert, bill, sms
from .exc import *
from .const import *




class Environment(object):
    REQUIRED_KEYS = ['CRYPT_AES_KEY', 'CRYPT_IMPAY_KEY', 'IMPAY_IV']
    PREFERRED_KEYS = ['CPCODE']

    def __init__(self, **kwargs):
        for key in self.REQUIRED_KEYS + self.PREFERRED_KEYS:
            setattr(self, key, kwargs.get(key, None))

    def validate(self):
        for key in self.REQUIRED_KEYS:
            if getattr(self, key) is None:
                raise ValueError(
                    'key {} is expected but not found.'.format(key))
        return True


default = Environment()




"""Replica of ImpayMember library."""

import hashlib
from base64 import b64encode as base64_encode, b64decode as base64_decode
from Crypto.Cipher import AES
from . import env


BLOCK_SIZE = 16


def sha256Encrypt(s):
    return hashlib.sha256(s).hex_digest()


def pad(s):
    length = len(s)
    pad_size = BLOCK_SIZE - (length % BLOCK_SIZE)
    padding = pad_size * chr(pad_size)
    return s + padding


def unpad(s):
    pad_size = ord(s[-1])
    return s[:-pad_size]


class Library(object):
    def __init__(self, env=env.default):
        self.env = env

    def encrypt_01(self, s):
        encrypted = self.encrypt_function(s)
        return encrypted

    def encrypt_function(self, s):
        encrypted = self.encrypt_block(pad(s))
        encoded = base64_encode(encrypted)
        return encoded

    # AES encrypt process
    def encrypt_block(self, plain_text, env=env.default):
        aes = AES.new(self.env.CRYPT_AES_KEY, AES.MODE_ECB)
        cipher_text = aes.encrypt(plain_text)
        return cipher_text

    def decrypt_01(self, s):
        decrypted = self.decrypt_function(s)
        return decrypted

    def decrypt_function(self, s):
        decoded = base64_decode(s)
        decrypted = self.decrypt_block(decoded)
        return decrypted

    def decrypt_block(self, cipher_text):
        aes = AES.new(self.env.CRYPT_AES_KEY, AES.MODE_ECB)
        plain_text = aes.decrypt(cipher_text)
        return unpad(plain_text)

    def decrypt_02(self, s):
        decrypted = self.impay_decrypt(s)
        return decrypted

    def impay_decrypt(self, s):
        decoded = s.decode('hex')
        aes = AES.new(
            self.env.CRYPT_IMPAY_KEY, AES.MODE_CBC, self.env.IMPAY_IV)
        plain_text = aes.decrypt(decoded)
        return unpad(plain_text)



from __future__ import with_statement

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def get_version():
    with open('impayident/version.txt') as f:
        return f.read().strip()


def get_readme():
    try:
        with open('README.rst') as f:
            return f.read().strip()
    except IOError:
        return ''


setup(
    name='impayident',
    version=get_version(),
    description='Impay Identification tool.',
    long_description=get_readme(),
    author='Jeong YunWon',
    author_email='jeong+impayident@youknowone.org',
    url='https://github.com/bpc/impayident',
    packages=(
        'impayident',
    ),
    package_data={
        'impayident': ['version.txt']
    },
    install_requires=[
        'six', 'requests', 'prettyexc',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],
)



# -*- coding: utf-8 -*-

from collections import namedtuple
import hashlib
import requests
from .exc import HttpException, ApiException

from . import lib
from . import env

from .const import *


class Request(object):
    URL = ''
    KEYS = []

    Response = None

    def __init__(self, **kwargs):
        self.env = kwargs.get('env', env.default)
        self.lib = lib.Library(self.env)
        kwargs = self._refine_init_args(kwargs)
        self.data = []
        for i, key in enumerate(self.KEYS):
            if key not in kwargs:
                raise TypeError(
                    u'key #{} {} is expected but not found'.format(i, key))
            datum = kwargs[key]
            setattr(self, key, datum)
            self.data.append(datum)

    def _refine_init_args(self, kwargs):
        raise NotImplementedError

    def __getitem__(self, index):
        try:
            intindex = int(index) - 1
            if intindex == len(self.KEYS) + 1:
                return self.CERTIFY
            return self.data[intindex]
        finally:
            return getattr(self, index)

    @property
    def string_data(self):
        return [u'{}'.format(datum) for datum in self.data]

    @property
    def CERTIFY(self):
        raw_string = u'/'.join(self.string_data)
        fucked = raw_string.encode('euc-kr')
        return hashlib.sha256(fucked).hexdigest()

    @property
    def cert_data(self):
        raw_string = '/'.join(self.string_data + [self.CERTIFY])
        fucked = raw_string.encode('euc-kr')
        return fucked

    def send(self):
        assert self.URL.startswith('http')
        encrypted = self.lib.encrypt_01(self.cert_data)
        response = requests.get(self.URL, {'cert_data': encrypted})
        if response.status_code != 200:
            raise HttpException(
                code=response.status_code, text=response.text,
                response=response)
        decrypted = self.lib.decrypt_01(response.text)
        unfucked = decrypted.decode('euc-kr')
        response = self.Response(*unfucked.split('/'))
        if response.RESULT != '0000':
            raise ApiException(
                code=response.RESULT, message=response.RESULTMSG,
                response=response)
        return response


CertResponse = namedtuple('CertResponse', [
    'CPCODE', 'CPORDERNO', 'CERTTYPE', 'TELECOM', 'NAME', 'PHONENO',
    'BIRTHDAY', 'SEX', 'KOREANFLAG', 'ACCESSFLAG', 'CERTTXID', 'RESULT',
    'SUBRET', 'RESULTMSG', 'CERTIFY'])


class CertRequest(Request):
    URL = 'https://cert.impay.co.kr/api/cert.do'
    KEYS = [
        'CPCODE', 'CPORDERNO', 'CERTTYPE', 'TELECOM', 'NAME', 'PHONENO',
        'BIRTHDAY', 'SEX', 'KOREANFLAG', 'ACCESSFLAG']
    Response = CertResponse

    def _refine_init_args(self, kwargs):
        if 'CPCODE' not in kwargs:
            kwargs['CPCODE'] = self.env.CPCODE
        if kwargs['TELECOM'] not in TELECOMS:
            raise ValueError
        if kwargs['SEX'] not in [MALE, FEMALE]:
            raise ValueError
        flags = kwargs['ACCESSFLAG']
        if isinstance(flags, list) or isinstance(flags, tuple):
            kwargs['ACCESSFLAG'] = ''.join(flags)
        if kwargs['KOREANFLAG'] not in [YES, NO]:
            raise ValueError
        for flag in kwargs['ACCESSFLAG']:
            if flag not in [YES, NO]:
                raise ValueError
        return kwargs


BillResponse = namedtuple('BillResponse', [
    'CPCODE', 'CPORDERNO', 'CERTTXID', 'PASSWORD', 'CI', 'DI', 'RESULT',
    'SUBRET', 'RESULTMSG', 'CERTIFY'])


class BillRequest(Request):
    URL = 'https://cert.impay.co.kr/api/bill.do'
    KEYS = ['CPCODE', 'CPORDERNO', 'CERTTXID', 'PASSWORD']

    Response = BillResponse

    def _refine_init_args(self, kwargs):
        if 'CPCODE' not in kwargs:
            kwargs['CPCODE'] = self.env.CPCODE
        if len(kwargs['PASSWORD']) != 6:
            raise ValueError
        return kwargs


SmsResponse = namedtuple('BillResponse', [
    'CPCODE', 'CPORDERNO', 'CERTTXID', 'RESULT', 'SUBRET', 'RESULTMSG',
    'CERTIFY'])


class SmsRequest(Request):
    URL = 'https://cert.impay.co.kr/api/sms.do'

    KEYS = ['CPCODE', 'CPORDERNO', 'CERTTXID']

    Response = SmsResponse

    def _refine_init_args(self, kwargs):
        if 'CPCODE' not in kwargs:
            kwargs['CPCODE'] = self.env.CPCODE
        return kwargs


def cert(**kwargs):
    """고객 식별 인증 요청"""
    return CertRequest(**kwargs).send()


def bill(**kwargs):
    """인증번호 인증 요청"""
    return BillRequest(**kwargs).send()


def sms(**kwargs):
    """SMS 재발송 요청"""
    return SmsRequest(**kwargs).send()
